<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\DeleteCascadeService;
use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;

class Indicator extends Crud {
    
    use TResponse;
    
	protected $dataSourceName = 'indicator';
    private $indicatorService;
    protected $organizationMongoManager;
    protected $goalsService;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService,
        \Service\Indicator $indicatorService, \Service\Goals $goalsService)
    {
        $this->indicatorService = $indicatorService;
        $this->organizationMongoManager = $organizationMongoManager;
        $this->goalsService = $goalsService;
        $this->organizationMongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
		parent::__construct($organizationMongoManager, $deleteCascadeService);
    }
    
    public function checkDrillDownIndicators(DTO\Request\Row $request, DTO\Response $response) {
        $indicatorsIds = $request->getRow()['indicators'];
        $response->addDataSet(new DataSet('checkDrillDownIndicators',
            $this->indicatorService->checkDrillDownIndicators($indicatorsIds)));
    }
    
    public function findNext(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        Utilities::convertDynamicFilterToArray($params);
        if (array_key_exists("in", $params)) {
            foreach ($params["in"] as $fieldName) {
                if ($params[$fieldName]) {
                    $params["dynamic_filter"][$fieldName] = ["relation"=>  "AND", "operation" => "EQUALS", "value"=> $params[$fieldName]];
                }
            }
        }
        $rows = $this->mongoManager->findNext($this->dataSourceName, $params, false);
        $total = $this->mongoManager->countNext($this->dataSourceName, $params, false);
        $dataSet = Utilities::parseDataSourceZeedhiNext($rows, $total);
        $dataSet['pagination']['page'] = array_key_exists('page', $params) ? intval($params['page']) : 1;
        $dataSet['pagination']['limit'] = array_key_exists('limit', $params) ? intval($params['limit']) : 99999;
        $this->sendResponse($dataSet, 200);
    }

	public function save(DTO\Request $request, DTO\Response $response){
        $indicator = !empty($request->getDataSet()->getRows()) ? $request->getDataSet()->getRows()[0] : null;
		if($indicator){
		    $this->updateDashboardIndicatorName($indicator);
            $dataBefore = null;
		    if(!empty($indicator['_id'])){
                $dataBefore = $this->indicatorService->findById($indicator['_id']);
            }
            if(!empty($dataBefore) && !empty($dataBefore["dataSource"]) && $dataBefore["dataSource"] != $indicator["dataSource"]){
                $this->indicatorService->updateDataSourceColumns($indicator['_id'],$indicator["dataSourceColumns"]);
            }
		    parent::save($request, $response);
            $indicatorId = $response->getDataSets()[0]->getRows()[0]['_id'];
            $this->indicatorService->setLog($dataBefore, $indicatorId);
		}
    }
    public function delete(DTO\Request $request, DTO\Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        $this->indicatorService->checkViewsWithIndicator($rows);
        $dataBefore = array();
        foreach ( $rows as $key => $row) {
            if(!empty($row['_id'])){
                $dataBefore[$key] = $this->indicatorService->findById($row['_id']);
            }
        }

        parent::delete($request, $response);

        foreach ( $rows as $key => $row) {
            if (!empty($row['_id'])) {
                $this->indicatorService->setLog($dataBefore[$key], $row['_id']);
            }
        }
    }

    protected function updateDashboardIndicatorName($indicator){
	    $dataSourceName = 'view';
	    $dashboardFilterCriteria = new FilterCriteria($dataSourceName);
	    $dashboardFilterCriteria->addCondition('type', 'D');
	    $dashboardsToUpdate = $this->organizationMongoManager->findBy($dashboardFilterCriteria)->getRows();

	    foreach ($dashboardsToUpdate as &$dashboard) {
	        if(isset($dashboard['indicators'])){
	            foreach ($dashboard['indicators'] as &$dashboardIndicator) {
	                if(isset($indicator['_id']) && $dashboardIndicator['indicator'] == $indicator['_id']){
	                    $dashboardIndicator['indicator_label'] = $indicator['name'];
	                }
	            }
	        }
	    }

	    $dataSet = new DataSet($dataSourceName, $dashboardsToUpdate);
	    $persistedRows = $this->organizationMongoManager->persist($dataSet);
	}

    public function getDefaultOrgIndicators(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $indicators = $this->indicatorService->getDataByFilterCriteria('indicator', $filterCriteria);
            $response->addDataSet(new DataSet($this->dataSourceName, $indicators));

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getDefaultOrgDataSources(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $indicators = $this->indicatorService->getDataByFilterCriteria('dataSource', $filterCriteria);
            $response->addDataSet(new DataSet('dataSource', $indicators));

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function importDefaultOrgIndicators(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            $indicatorsId = $row['indicators'];
            if(!empty($indicatorsId)) {
                $importedItems = [];
                foreach($indicatorsId as $indicator){
                    $importedNow = $this->indicatorService->saveDefaultOrgIndicatorsAndDataSources(array($indicator));
                    if(isset($importedItems['indicator']) && isset($importedNow['indicator'])){
                        $importedItems['indicator'] = array_merge($importedItems['indicator'], $importedNow['indicator']);
                        $importedItems['datasource'] = array_merge($importedItems['datasource'], $importedNow['datasource']);
                        $importedItems['widget'] = array_merge($importedItems['widget'], $importedNow['widget']);
                    }else{
                        $importedItems = $importedNow;
                    }
                }
                $this->indicatorService->setAllDrillDowns($importedItems);
                
                $response->addDataSet(new DataSet('indicator', $importedItems['indicator']));
                $response->addDataSet(new DataSet('datasource', $importedItems['datasource']));
                $response->addDataSet(new DataSet('widget', $importedItems['widget']));
                $this->indicatorService->setLogImportation($importedItems);
                $nrImportedIndicators = count($importedItems['indicator']);
                $response->addNotification(new DTO\Response\Notification("{$nrImportedIndicators} indicator(s) persisted with success"));
            } else {
                $response->setError(new DTO\Response\Error("Invalid parameters. Missing the indicators", 500));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function dataSourceColumnsDimension(DTO\Request\Filter $request, DTO\Response $response) {
        $dimensions = [];
        $widgetFilterCriteria = new FilterCriteria($this->dataSourceName);
        $indicator = $request->getFilterCriteria()->getConditions()[0]["value"];
        $widgetFilterCriteria->addCondition("_id", $indicator);
        $widgetsColumns = $this->organizationMongoManager->findBy($widgetFilterCriteria)->getRows();
        $dataSourceColumns = isset($widgetsColumns[0]["dataSourceColumns"]) ? $widgetsColumns[0]["dataSourceColumns"] : [];

        foreach ($dataSourceColumns as $column) {
            if (isset($column["type"]) && $column["type"] === "dimension" && $column["isDimension"]){
                $dimensions[] = $column;
            }
        }

        if(isset($dimensions)){
            $response->addDataSet(new DataSet($this->dataSourceName, $dimensions));
        } else {
            $response->addDataSet(new DataSet($this->dataSourceName, []));
        }

    }

    public function dataSourceColumnsFact(DTO\Request\Filter $request, DTO\Response $response) {
        $facts = [];
        $widgetFilterCriteria = new FilterCriteria($this->dataSourceName);
        $indicator = $request->getFilterCriteria()->getConditions()[0]["value"];
        $widgetFilterCriteria->addCondition("_id", $indicator);
        $widgetsColumns = $this->organizationMongoManager->findBy($widgetFilterCriteria)->getRows();
        $dataSourceColumns = $widgetsColumns[0]["dataSourceColumns"];

        foreach ($dataSourceColumns as $column) {
            if (isset($column["type"]) && $column["type"] === "fact" && $column["isFact"]){
                $facts[] = $column;
            }
        }
        if(isset($facts)){
            $response->addDataSet(new DataSet($this->dataSourceName, $facts));
        } else{
            $response->addDataSet(new DataSet($this->dataSourceName, []));
        }
    }

    public function getViewDependencies(DTO\Request $request, DTO\Response $response){
        $request = $this->buildDataSetIfRow($request);
        $dataSet = $request->getDataSet()->getRows();
        $views = $dataSet[0]["views"];
        $dependencies = [];
        foreach($views as $view){
            $dependentGoals = $this->goalsService->getGoalsByViewId($view['_id']);
            array_push($dependencies, $dependentGoals);
        }
        $response->addDataSet(new DataSet('result', $dependencies));
    }

    protected function buildDataSetIfRow(DTO\Request $request) {
		if ($request instanceof DTO\Request\Row) {
			$dataSet = new DataSet($this->dataSourceName, array($request->getRow()));
			$request = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
		}
		return $request;
	}
}
